package org.jboss.seam.example.selectitems;

import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.faces.application.FacesMessage;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.Begin;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.core.FacesMessages;
import org.jboss.seam.example.selectitems.Client.Discount;
import org.jboss.seam.log.Log;
import org.jboss.seam.selectitems.annotations.SelectItems;

@Stateful
@Name("clientAction")
@Interceptors(org.jboss.seam.ejb.SeamInterceptor.class)
public class ClientActionBean implements ClientAction {
	
	@In(required=false)
	@Out
	private Client client;
	
	@SelectItems(label="description", addNoSelectionLabel="Please select...")
	private List<ClientType> clientTypes;
	
	@SelectItems(label="description")
	private List<Branch> branches;
	
	@SuppressWarnings("unused")
	@SelectItems(label="labelKey") 
	private List<Discount> discounts;
	
	@In(create=true)
	private EntityManager em;
	
	@Logger
	private Log log;
	
	@In(create=true)
	private FacesMessages facesMessages;
	

	@Create
	@Begin
	public void begin() {		
		client = em.find(Client.class, 1);
	}
	

	@Remove @Destroy
	public void destroy() {
		
	}
	

	@SuppressWarnings("unchecked")
	@Factory("clientTypes")
	public void buildClientTypes() {
		clientTypes = em.createQuery("select ct from ClientType ct").getResultList();
		log.info("Client type list has " + clientTypes.size() + " entries");
	}
	
	public void apply() {
		try {
			facesMessages.add(FacesMessage.SEVERITY_INFO, "Changes applied");
		} catch (Exception e) {
			facesMessages.add(FacesMessage.SEVERITY_ERROR, "An Exception occurred, look at the console");
			e.printStackTrace();
		}
		
	}

	@SuppressWarnings("unchecked")
	@Factory("branches")
	public void buildBranches() {
		branches = em.createQuery("select b from Branch b").getResultList();
		log.info("Branch list has " + branches.size() + " entries");
		
	}
	
	@Factory("discounts")
	public void buildDiscounts() {
		discounts = Client.Discount.valuesAsList();
		log.info("Discount list has " + discounts.size() + " entries");
	}

}
