package org.jboss.seam.selectitems.databinding;

import java.lang.reflect.Method;
import java.util.List;

import javax.faces.model.SelectItem;

import org.jboss.seam.ScopeType;
import org.jboss.seam.core.Messages;
import org.jboss.seam.databinding.DataBinder;
import org.jboss.seam.selectitems.SelectItemsConfig;
import org.jboss.seam.selectitems.annotations.SelectItems;
import org.jboss.seam.selectitems.annotations.SelectItems.Strategy;
import org.jboss.seam.selectitems.jsf.SelectItemsListModel;
import org.jboss.seam.util.Reflections;

/**
 * SelectItems binder for JBoss Seam. See SelectItems for details
 * 
 * @author Peter Muir
 * @link org.jboss.seam.databinding.selectitems.SelectItems
 */
public class SelectItemsBinder implements
		DataBinder<SelectItems, List, List<SelectItem>> {

	public String getVariableName(SelectItems out) {
		return out.value();
	}

	public ScopeType getVariableScope(SelectItems out) {
		return out.scope();
	}

	/**
	 * Outject the wrapped data, using the strategy requested
	 */
	public List<SelectItem> wrap(SelectItems out, List value) {
		return buildSelectItems(out, value);
	}

	public List getWrappedData(SelectItems out, List<SelectItem> wrapper) {
		return ((SelectItemsListModel<SelectItem>) wrapper).getWrappedData();
	}

	public Object getSelection(SelectItems out, List<SelectItem> wrapper) {
		return null;
	}

	public boolean isDirty(SelectItems out, List<SelectItem> wrapper, List value) {
		return !((SelectItemsListModel<SelectItem>) wrapper).getWrappedData()
				.equals(value);
	}

	private List<SelectItem> buildSelectItems(SelectItems out, List value) {
		if (out.strategy().equals(Strategy.INDEX)) {
			return indexStrategy(out, value);
		} else if (out.strategy().equals(Strategy.STRING)) {
			return stringStrategy(out, value);
		} else if (out.strategy().equals(Strategy.OBJECT)) {
			return objectStrategy(out, value);
		}
		return null;
	}

	private SelectItemsListModel<SelectItem> indexStrategy(SelectItems out,
			List value) {
		// Use SelectItemsListModel so we can store the orignal data
		SelectItemsListModel<SelectItem> selectItems = new SelectItemsListModel<SelectItem>();
		selectItems.setWrappedData(value);

		if (!"".equals(out.addNoSelectionLabel())) {
			selectItems.add(new SelectItem("-1", getMessage(out
					.addNoSelectionLabel(), out), "", false));
		}
		int i = 0;
		for (Object o : value) {
			selectItems.add(new SelectItem(i + "", getLabel(out, o), "",
					getDisabled(out, o)));
			i++;
		}
		return selectItems;
	}

	private SelectItemsListModel<SelectItem> objectStrategy(SelectItems out,
			List value) {
		// Use SelectItemsListModel so we can store the orignal data
		SelectItemsListModel<SelectItem> selectItems = new SelectItemsListModel<SelectItem>();
		selectItems.setWrappedData(value);
		if (!"".equals(out.addNoSelectionLabel())) {
			selectItems.add(new SelectItem(
					SelectItemsConfig.NO_SELECTION_VALUE, getMessage(out
							.addNoSelectionLabel(), out), "", false));
		}
		for (Object o : value) {
			selectItems.add(new SelectItem(o, getLabel(out, o), "",
					getDisabled(out, o)));
		}
		return selectItems;
	}

	private SelectItemsListModel<SelectItem> stringStrategy(SelectItems out,
			List value) {
		// Use SelectItemsListModel so we can store the orignal data
		SelectItemsListModel<SelectItem> selectItems = new SelectItemsListModel<SelectItem>();
		selectItems.setWrappedData(value);
		if (!"".equals(out.addNoSelectionLabel())) {
			selectItems.add(new SelectItem(
					SelectItemsConfig.NO_SELECTION_VALUE, getMessage(out
							.addNoSelectionLabel(), out), "", false));
		}

		for (Object o : value) {
			selectItems.add(new SelectItem(getJsfValue(out, o),
					getLabel(out, o), "", getDisabled(out, o)));
		}
		return selectItems;
	}

	private String getLabel(SelectItems out, Object bean) {
		if (out.label().equals("")) {
			return bean.toString();
		} else {
			String key = getValue(out.label(), bean).toString();
			return getMessage(key, out);
		}
	}

	private Boolean getDisabled(SelectItems out, Object bean) {
		if (!out.disabled().equals("")) {
			return (Boolean) getValue(out.disabled(), bean);
		} else {
			return false;
		}
	}

	private String getJsfValue(SelectItems out, Object bean) {
		if (out.jsfValue().equals("")) {
			return bean.toString();
		} else {
			return (String) getValue(out.jsfValue(), bean);
		}
	}

	private Object getValue(String property, Object bean) {
		try {
			return Reflections.invoke(getMethod(property, bean.getClass()),
					bean);
		} catch (Exception e) {
			throw new RuntimeException("Unable to access " + property
					+ " property", e);
		}
	}

	private Method getMethod(String property, Class clazz)
			throws SecurityException, NoSuchMethodException {
		property = property.substring(0, 1).toUpperCase()
				+ property.substring(1);
		Class[] sig = {};
		try {
			return clazz.getMethod("get" + property, sig);
		} catch (NoSuchMethodException e) {
			return clazz.getMethod("is" + property, sig);
		}
	}

	private String getMessage(String key, SelectItems out) {
		if (out.useMessageBundle()) {
			try {
				return Messages.instance().get(key).toString();
			} catch (Exception e) {
				return key;
			}
		} else {
			return key;
		}
	}

}
