package org.jboss.seam.selectitems.jsf;

import java.util.List;

import javax.faces.FacesException;
import javax.faces.component.UISelectItems;
import javax.faces.model.SelectItem;

import org.jboss.cache.CacheException;
import org.jboss.seam.core.PojoCache;
import org.jboss.seam.log.Log;
import org.jboss.seam.log.LogImpl;
import org.jboss.seam.selectitems.SelectItemsConfig;

import com.sun.facelets.FaceletContext;
import com.sun.facelets.tag.MetaRuleset;
import com.sun.facelets.tag.TagAttribute;
import com.sun.facelets.tag.jsf.ComponentConfig;
import com.sun.facelets.tag.jsf.ComponentHandler;

public class SelectItemsComponentHandler extends ComponentHandler {
	
	private Log log = new LogImpl(SelectItemsComponentHandler.class);
	private TagAttribute cacheKeyAttribute;
	private TagAttribute cacheRegionAttribute;
	
	public SelectItemsComponentHandler(ComponentConfig config) {
		super(config);
		this.cacheKeyAttribute = this.getAttribute("cacheKey");
		this.cacheRegionAttribute = this.getAttribute("cacheRegion");
	}
	
	@Override
	protected void setAttributes(FaceletContext ctx, Object instance) {
		super.setAttributes(ctx, instance);
		if (instance instanceof UISelectItems) {
			UISelectItems cmp = (UISelectItems) instance;
			String key = evaluateCacheKey(ctx);
			if (key != null) {
				String region = evaluateCacheRegion(ctx);
				List<SelectItem> cachedContent = getFromCache(key, region);
				// TODO How does all this impact performance?
				if (cachedContent != null) {
					log.debug("retrieving selectitems from cache: " + key);
					cmp.setValue(cachedContent);
				} else {
					// Execute value binding now and place content in cache
					Object value = cmp.getValue();
					try {
						List<SelectItem> selectItems = (List<SelectItem>) value;
						putInCache(key, region, selectItems);
						cmp.setValue(selectItems);
					} catch (Exception e) {
						throw new FacesException("value must be of type List<SelectItems>");
					}
				}
			}
		}
	}
	
	@Override
	protected MetaRuleset createMetaRuleset(Class type) {
		return super.createMetaRuleset(type).ignore("cacheKey").ignore("cacheRegion");
	}
	
	@SuppressWarnings("unchecked")
	private List<SelectItem> getFromCache(String key, String region) {
		try {
			return (List<SelectItem>) PojoCache.instance().get(region, key);
		} catch (CacheException ce) {
			log.error("error accessing cache", ce);
			return null;
		}
	}

	private String evaluateCacheKey(FaceletContext ctx) {
		if (cacheKeyAttribute != null && !cacheKeyAttribute.getValue(ctx).equals("")) {
			return "org.jboss.seam.selectitems.cache." + cacheKeyAttribute.getValue(ctx);
		} else {
			return null;
		}
	}

	private void putInCache(String key, String region, List<SelectItem> content) {
		try {
			PojoCache.instance().put(region, key, content);
		} catch (CacheException ce) {
			log.error("error accessing cache", ce);
		}
	}

	private String evaluateCacheRegion(FaceletContext ctx) {
		if (cacheRegionAttribute != null && !cacheRegionAttribute.getValue(ctx).equals("")) {
			return cacheRegionAttribute.getValue(ctx);
		} else {
			return SelectItemsConfig.CACHE_REGION;
		}
	}
	
	

}
